/*
 * The Yices SMT Solver. Copyright 2014 SRI International.
 *
 * This program may only be used subject to the noncommercial end user
 * license agreement which is downloadable along with this program.
 */

/*
 * Tables for parsing the SMT-LIB 2.0 language
 */

#ifndef __SMT2_PARSE_TABLES_H
#define __SMT2_PARSE_TABLES_H

#include <stdint.h>

/*
 * States
 */
typedef enum state_s {
  c0, c1, c3, c4, c5, c6, c6a, c8, c9, c9a, c9b,
  c10, c10a, c10b, c11, c11a, c11b, c11d, c11f, c12, c12b, c13,
  a0, a1, v0,
  s0, s1, s2, s3, s4, s5, s6, s7, s8, s10,
  t0, t1, t2, t2a, t2b, t2d, t2e,
  t3, t3a, t3b, t3d, t3e,
  t4a, t4b, t4c, t4d, t4e, t4g,
  t5, t5a, t5b, t5c, t5d,
  t6, t6a, t6b, t6c, t6d, t6e, t6g, t6h, t6i, t6j,
  t7, t7a, t7b, t8a,
  r0,
} state_t;


/*
 * Action codes
 */
typedef enum actions {
  // commands
  next_goto_c1,
  empty_command_return,
  check_sat_next_goto_r0,
  get_assertions_next_goto_r0,
  get_proof_next_goto_r0,
  get_unsat_core_next_goto_r0,
  get_assignment_next_goto_r0,
  exit_next_goto_r0,
  push_next_goto_c3,
  pop_next_goto_c3,
  get_option_next_goto_c4,
  get_info_next_goto_c4,
  set_logic_next_goto_c5,
  set_option_next_goto_c6,
  set_info_next_goto_c6,
  assert_next_push_r0_goto_t0,
  declare_sort_next_goto_c8,
  define_sort_next_goto_c9,
  declare_fun_next_goto_c10,
  define_fun_next_goto_c11,
  get_value_next_goto_c12,
  get_model_next_goto_r0,
  echo_next_goto_c13,
  reset_next_goto_r0,

  // arguments to the commands
  numeral_next_goto_r0,
  keyword_next_goto_r0,
  symbol_next_goto_r0,
  keyword_next_goto_c6a,
  next_return,
  push_r0_goto_a0,
  symbol_next_goto_c3,
  symbol_next_goto_c9a,
  next_goto_c9b,
  next_push_r0_goto_s0,
  symbol_next_goto_c9b,
  symbol_next_goto_c10a,
  next_goto_c10b,
  push_c10b_goto_s0,
  symbol_next_goto_c11a,
  next_goto_c11b,
  next_push_r0_push_t0_goto_s0,
  next_goto_c11d,
  symbol_next_push_c11f_goto_s0,
  eval_next_goto_c11b,
  next_push_c12b_goto_t0,
  next_goto_r0,
  push_c12b_goto_t0,
  string_next_goto_r0,

  // attribute values + s-expressions
  numeral_next_return,
  decimal_next_return,
  hexadecimal_next_return,
  binary_next_return,
  string_next_return,
  symbol_next_return,
  next_goto_a1,
  push_a1_goto_v0,
  keyword_next_return,

  // sorts
  sort_symbol_next_return,
  next_goto_s1,
  next_goto_s2,
  next_goto_s5,
  symbol_next_push_s10_goto_s0,
  symbol_next_goto_s3,
  numeral_next_goto_s4,
  next_goto_s6,
  symbol_next_goto_s7,
  numeral_next_goto_s8,
  next_push_s10_goto_s0,
  push_s10_goto_s0,

  // terms
  term_symbol_next_return,
  next_goto_t1,
  next_goto_t2,           // (let
  forall_next_goto_t3,    // (forall
  exists_next_goto_t3,    // (exists
  next_push_t4a_goto_t0,  // (!
  next_goto_t5,           // (as
  next_goto_t6,           // ((
  next_goto_t7,           // (_

  // simple function application (<symbol> <term> ... <term>)
  symbol_next_push_t8a_goto_t0,

  // (let ...
  next_goto_t2a,
  next_goto_t2b,
  symbol_next_push_t2d_goto_t0,
  next_goto_t2e,
  next_push_r0_goto_t0,

  // (exists ... and (forall ...
  next_goto_t3a,
  next_goto_t3b,
  symbol_next_push_t3d_goto_s0,
  next_goto_t3e,

  // (! <term> ...
  check_keyword_then_branch,
  push_t4c_goto_a0,
  symbol_next_goto_t4c,
  next_push_t4g_goto_t0,
  next_goto_t4c,
  push_t4g_goto_t0,

  // (as ...
  next_goto_t5a,
  symbol_next_push_r0_goto_s0,
  next_goto_t5b,
  symbol_next_goto_t5c,
  numeral_next_goto_t5d,

  // (( ...
  next_goto_t6a,
  next_goto_t6h,

  // ((as ...
  next_goto_t6b,
  symbol_next_push_t6g_goto_s0,
  next_goto_t6c,
  symbol_next_goto_t6d,
  numeral_next_goto_t6e,
  next_push_t6g_goto_s0,
  next_push_t8a_goto_t0,

  // ((_ ...
  symbol_next_goto_t6i,
  numeral_next_goto_t6j,

  // (_ ...
  symbol_next_goto_t7a,
  numeral_next_goto_t7b,

  // after <term> in a function application
  push_t8a_goto_t0,

  // errors
  error_lp_expected,
  error_string_expected,
  error_symbol_expected,
  error_numeral_expected,
  error_keyword_expected,
  error_rp_expected,
  error_underscore_expected,
  error_command_expected,
  error,
} smt2_action_t;


/*
 * Tables generated by table_builder
 * see utils/table_builder.c
 */

// Table sizes
#define NSTATES 73
#define BSIZE 272

// Default values for each state
static const uint8_t default_value[NSTATES] = {
  error_lp_expected,
  error_command_expected,
  error_numeral_expected,
  error_keyword_expected,
  error_symbol_expected,
  error_keyword_expected,
  push_r0_goto_a0,
  error_symbol_expected,
  error_symbol_expected,
  error_lp_expected,
  error,
  error_symbol_expected,
  error_lp_expected,
  push_c10b_goto_s0,
  error_symbol_expected,
  error_lp_expected,
  error,
  error_symbol_expected,
  error_rp_expected,
  error_lp_expected,
  push_c12b_goto_t0,
  error,
  error,
  push_a1_goto_v0,
  error,
  error,
  error,
  error_symbol_expected,
  error_numeral_expected,
  error,
  error_underscore_expected,
  error_symbol_expected,
  error_numeral_expected,
  error,
  push_s10_goto_s0,
  error,
  error,
  error_lp_expected,
  error_lp_expected,
  error_symbol_expected,
  error_rp_expected,
  error,
  error_lp_expected,
  error_lp_expected,
  error_symbol_expected,
  error_rp_expected,
  error,
  error_keyword_expected,
  push_t4c_goto_a0,
  error,
  error_symbol_expected,
  error_lp_expected,
  push_t4g_goto_t0,
  error,
  error_underscore_expected,
  error_symbol_expected,
  error_numeral_expected,
  error,
  error,
  error,
  error_underscore_expected,
  error_symbol_expected,
  error_numeral_expected,
  error,
  error_rp_expected,
  error_symbol_expected,
  error_numeral_expected,
  error,
  error_symbol_expected,
  error_numeral_expected,
  error,
  push_t8a_goto_t0,
  error_rp_expected,
};

// Base values for each state
static const uint8_t base[NSTATES] = {
     0,   0,   0,   0,   3,   3,   0,   6,   9,   4,
    51,  45,   5,   5,  48,   7,   8,  54,  15,  19,
    19,  48,  61,  57,  71,  97, 100,  90,  69,  81,
    58, 108,  80, 117, 103, 119, 136, 107, 110, 125,
   113, 146, 121, 129, 149, 134, 162, 133, 163, 169,
   160, 171, 171, 174, 160, 177, 177, 191, 178, 188,
   166, 195, 181, 204, 186, 200, 195, 209, 212, 196,
   210, 205, 221,
};

// Check table
static const uint8_t check[BSIZE] = {
     0,   6,   0,   2,   9,  12,  13,  15,  16,  16,
     3,   4,   4,   5,   7,   7,  18,   8,   8,  19,
    20,   1,   1,   1,   1,   1,   1,   1,   1,   1,
     1,   1,   1,   1,   1,   1,   1,   1,   1,   1,
     1,   1,   1,   4,   4,   4,   7,   7,   7,   8,
     8,   8,  10,  11,  11,  21,  14,  14,  23,  10,
    10,  22,  17,  17,  22,  22,  22,  22,  22,  22,
    22,  24,  28,  30,  24,  24,  24,  24,  24,  24,
    24,  24,  29,  32,  29,  11,  11,  11,  14,  14,
    14,  10,  10,  10,  17,  17,  17,  25,  27,  27,
    26,  22,  22,  22,  34,  25,  25,  37,  26,  26,
    38,  24,  24,  24,  40,  26,  31,  31,  33,  35,
    33,  42,  35,  35,  35,  35,  35,  35,  35,  43,
    27,  27,  27,  39,  39,  45,  36,  25,  25,  25,
    26,  26,  26,  47,  36,  36,  41,  41,  31,  31,
    31,  36,  36,  36,  36,  36,  36,  44,  44,  35,
    35,  35,  46,  46,  48,  39,  39,  39,  50,  50,
    49,  51,  52,  48,  53,  54,  36,  36,  36,  49,
    56,  60,  53,  53,  62,  55,  55,  64,  59,  44,
    44,  44,  57,  58,  57,  58,  59,  59,  66,  69,
    50,  50,  50,  61,  61,  63,  71,  63,  65,  65,
    67,  70,  67,  70,  53,  53,  53,  55,  55,  55,
    68,  68,  72,  73,  73,  73,  73,  73,  59,  59,
    59,  73,  73,  73,  73,  61,  61,  61,  73,  73,
    65,  65,  65,  73,  73,  73,  73,  73,  73,  73,
    73,  73,  68,  68,  68,  73,  73,  73,  73,  73,
    73,  73,  73,  73,  73,  73,  73,  73,  73,  73,
    73,  73,
};

// Value table
static uint8_t value[BSIZE] = {
  next_goto_c1,
  next_return,
  empty_command_return,
  numeral_next_goto_r0,
  next_goto_c9b,
  next_goto_c10b,
  next_push_r0_goto_s0,
  next_goto_c11b,
  next_goto_c11d,
  next_push_r0_push_t0_goto_s0,
  keyword_next_goto_r0,
  symbol_next_goto_r0,
  symbol_next_goto_r0,
  keyword_next_goto_c6a,
  symbol_next_goto_c3,
  symbol_next_goto_c3,
  eval_next_goto_c11b,
  symbol_next_goto_c9a,
  symbol_next_goto_c9a,
  next_push_c12b_goto_t0,
  next_goto_r0,
  assert_next_push_r0_goto_t0,
  check_sat_next_goto_r0,
  declare_sort_next_goto_c8,
  declare_fun_next_goto_c10,
  define_sort_next_goto_c9,
  define_fun_next_goto_c11,
  exit_next_goto_r0,
  get_assertions_next_goto_r0,
  get_assignment_next_goto_r0,
  get_info_next_goto_c4,
  get_option_next_goto_c4,
  get_proof_next_goto_r0,
  get_unsat_core_next_goto_r0,
  get_value_next_goto_c12,
  pop_next_goto_c3,
  push_next_goto_c3,
  set_logic_next_goto_c5,
  set_info_next_goto_c6,
  set_option_next_goto_c6,
  get_model_next_goto_r0,
  echo_next_goto_c13,
  reset_next_goto_r0,
  symbol_next_goto_r0,
  symbol_next_goto_r0,
  symbol_next_goto_r0,
  symbol_next_goto_c3,
  symbol_next_goto_c3,
  symbol_next_goto_c3,
  symbol_next_goto_c9a,
  symbol_next_goto_c9a,
  symbol_next_goto_c9a,
  next_push_r0_goto_s0,
  symbol_next_goto_c10a,
  symbol_next_goto_c10a,
  string_next_goto_r0,
  symbol_next_goto_c11a,
  symbol_next_goto_c11a,
  next_return,
  symbol_next_goto_c9b,
  symbol_next_goto_c9b,
  next_goto_a1,
  symbol_next_push_c11f_goto_s0,
  symbol_next_push_c11f_goto_s0,
  numeral_next_return,
  decimal_next_return,
  hexadecimal_next_return,
  binary_next_return,
  string_next_return,
  symbol_next_return,
  symbol_next_return,
  next_goto_a1,
  numeral_next_goto_s4,
  next_goto_s6,
  numeral_next_return,
  decimal_next_return,
  hexadecimal_next_return,
  binary_next_return,
  string_next_return,
  symbol_next_return,
  symbol_next_return,
  keyword_next_return,
  next_return,
  numeral_next_goto_s8,
  numeral_next_goto_s4,
  symbol_next_goto_c10a,
  symbol_next_goto_c10a,
  symbol_next_goto_c10a,
  symbol_next_goto_c11a,
  symbol_next_goto_c11a,
  symbol_next_goto_c11a,
  symbol_next_goto_c9b,
  symbol_next_goto_c9b,
  symbol_next_goto_c9b,
  symbol_next_push_c11f_goto_s0,
  symbol_next_push_c11f_goto_s0,
  symbol_next_push_c11f_goto_s0,
  next_goto_s1,
  symbol_next_goto_s3,
  symbol_next_goto_s3,
  next_goto_s5,
  symbol_next_return,
  symbol_next_return,
  symbol_next_return,
  next_return,
  sort_symbol_next_return,
  sort_symbol_next_return,
  next_goto_t2a,
  symbol_next_push_s10_goto_s0,
  symbol_next_push_s10_goto_s0,
  next_goto_t2b,
  symbol_next_return,
  symbol_next_return,
  symbol_next_return,
  next_goto_t2e,
  next_goto_s2,
  symbol_next_goto_s7,
  symbol_next_goto_s7,
  next_push_s10_goto_s0,
  next_goto_t1,
  numeral_next_goto_s8,
  next_goto_t3a,
  numeral_next_return,
  decimal_next_return,
  hexadecimal_next_return,
  binary_next_return,
  string_next_return,
  term_symbol_next_return,
  term_symbol_next_return,
  next_goto_t3b,
  symbol_next_goto_s3,
  symbol_next_goto_s3,
  symbol_next_goto_s3,
  symbol_next_push_t2d_goto_t0,
  symbol_next_push_t2d_goto_t0,
  next_goto_t3e,
  next_goto_t6,
  sort_symbol_next_return,
  sort_symbol_next_return,
  sort_symbol_next_return,
  symbol_next_push_s10_goto_s0,
  symbol_next_push_s10_goto_s0,
  symbol_next_push_s10_goto_s0,
  check_keyword_then_branch,
  symbol_next_push_t8a_goto_t0,
  symbol_next_push_t8a_goto_t0,
  next_goto_t2b,
  next_push_r0_goto_t0,
  symbol_next_goto_s7,
  symbol_next_goto_s7,
  symbol_next_goto_s7,
  next_goto_t7,
  next_push_t4a_goto_t0,
  next_goto_t5,
  next_goto_t2,
  exists_next_goto_t3,
  forall_next_goto_t3,
  symbol_next_push_t3d_goto_s0,
  symbol_next_push_t3d_goto_s0,
  term_symbol_next_return,
  term_symbol_next_return,
  term_symbol_next_return,
  next_goto_t3b,
  next_push_r0_goto_t0,
  next_return,
  symbol_next_push_t2d_goto_t0,
  symbol_next_push_t2d_goto_t0,
  symbol_next_push_t2d_goto_t0,
  symbol_next_goto_t4c,
  symbol_next_goto_t4c,
  next_return,
  next_push_t4g_goto_t0,
  next_goto_t4c,
  check_keyword_then_branch,
  next_goto_t5a,
  next_goto_t5b,
  symbol_next_push_t8a_goto_t0,
  symbol_next_push_t8a_goto_t0,
  symbol_next_push_t8a_goto_t0,
  check_keyword_then_branch,
  numeral_next_goto_t5d,
  next_goto_t6c,
  symbol_next_push_r0_goto_s0,
  symbol_next_push_r0_goto_s0,
  numeral_next_goto_t6e,
  symbol_next_goto_t5c,
  symbol_next_goto_t5c,
  next_push_t8a_goto_t0,
  next_goto_t6b,
  symbol_next_push_t3d_goto_s0,
  symbol_next_push_t3d_goto_s0,
  symbol_next_push_t3d_goto_s0,
  next_push_r0_goto_s0,
  next_goto_t6h,
  numeral_next_goto_t5d,
  next_goto_t6a,
  symbol_next_push_t6g_goto_s0,
  symbol_next_push_t6g_goto_s0,
  numeral_next_goto_t6j,
  numeral_next_goto_t7b,
  symbol_next_goto_t4c,
  symbol_next_goto_t4c,
  symbol_next_goto_t4c,
  symbol_next_goto_t6d,
  symbol_next_goto_t6d,
  next_push_t6g_goto_s0,
  next_return,
  numeral_next_goto_t6e,
  symbol_next_goto_t6i,
  symbol_next_goto_t6i,
  next_push_t8a_goto_t0,
  next_return,
  numeral_next_goto_t6j,
  numeral_next_goto_t7b,
  symbol_next_push_r0_goto_s0,
  symbol_next_push_r0_goto_s0,
  symbol_next_push_r0_goto_s0,
  symbol_next_goto_t5c,
  symbol_next_goto_t5c,
  symbol_next_goto_t5c,
  symbol_next_goto_t7a,
  symbol_next_goto_t7a,
  next_return,
  error,
  error,
  error,
  error,
  error,
  symbol_next_push_t6g_goto_s0,
  symbol_next_push_t6g_goto_s0,
  symbol_next_push_t6g_goto_s0,
  error,
  error,
  error,
  error,
  symbol_next_goto_t6d,
  symbol_next_goto_t6d,
  symbol_next_goto_t6d,
  error,
  error,
  symbol_next_goto_t6i,
  symbol_next_goto_t6i,
  symbol_next_goto_t6i,
  error,
  error,
  error,
  error,
  error,
  error,
  error,
  error,
  error,
  symbol_next_goto_t7a,
  symbol_next_goto_t7a,
  symbol_next_goto_t7a,
  error,
  error,
  error,
  error,
  error,
  error,
  error,
  error,
  error,
  error,
  error,
  error,
  error,
  error,
  error,
  error,
  error,
};


#endif /* __SMT2_PARSE_TABLES_H */
